package de.ullisroboterseite.ursai2notification;

import com.google.appinventor.components.annotations.*;
import com.google.appinventor.components.annotations.androidmanifest.*;
import com.google.appinventor.components.common.*;
import com.google.appinventor.components.runtime.*;
import com.google.appinventor.components.runtime.util.*;

import android.os.Build;
import android.os.Handler;
import android.app.Notification.*;
import android.app.NotificationManager;
import android.graphics.Bitmap;
import android.content.*;
import java.util.*;

// Macht die Sache etwas übersichtlicher
@SimpleObject()
public class NotificationProperties extends AndroidNonvisibleComponent {
   static final String LOG_TAG = ChannelProperties.LOG_TAG;
   static final String DELETE_TAG = "URS_NOTIFICATION_DELETE";
   static final String ID_TAG = "NumberID";
   static final String EVENT_TAG = "URS_NOTIFICATION_EVENT";

   final Handler handler = new Handler();

   // Property Fields
   int numberID = 1;
   boolean autoCancel = false;
   int smallIcon = 0x1080095;
   String smallSysemIconName = "ic_launcher";
   String smallIconPath = "";
   String title = "";
   String text = "";
   String largeIconPath = "";
   String bigPicturePath = "";
   int maxProgress = 100;

   long when = 0;
   boolean showWhen = false;
   boolean usesChronometer = false;
   boolean chronometerCountDown = false;

   int color = Component.COLOR_DEFAULT;
   boolean colorized = false;

   boolean flagNoClear = false;
   boolean flagInsistent = false;
   boolean flagOngoingEvent = false;
   boolean flagOnlyAlertOnce = false;
   boolean cancelOnDestroy = false;

   UrsAI2NotificationChannel destroyChannel = null;
   UrsNotification destroyNotification = null;
   UrsIntent destroyIntent = null;

   List<Action> actions = new ArrayList<Action>();

   NotificationManager notificationManager;

   public NotificationProperties(ComponentContainer container) {
      super(container.$form());
      notificationManager = (NotificationManager) form.getSystemService(Context.NOTIFICATION_SERVICE);
   }

   @SimpleProperty(description = "The SDK version of the software currently running on this hardware device.")
   public int VersionSDK() {
      return Build.VERSION.SDK_INT;
   }

   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_NON_NEGATIVE_INTEGER, defaultValue = "1")
   @SimpleProperty(description = "An identifier for this notification unique within the application.")
   public void NumberID(int value) {
      numberID = value;
   }

   @SimpleProperty(description = "An identifier for this notification unique within the application.")
   public int NumberID() {
      return numberID;
   }

   // https://developer.android.com/reference/android/app/Notification.Builder#setAutoCancel(boolean)
   // ab API Level 11
   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
   @SimpleProperty(description = "The notification is automatically canceled when the user clicks it in the panel.")
   public void AutoCancel(boolean value) {
      autoCancel = value;
   }

   @SimpleProperty(description = "The notification is automatically canceled when the user clicks it in the panel.")
   public boolean AutoCancel() {
      return autoCancel;
   }

   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
   @SimpleProperty(description = "The notification is automatically canceled when the app is destroyed.")
   public void CancelOnDestroy(boolean value) {
      cancelOnDestroy = value;
   }

   @SimpleProperty(description = "The notification is automatically canceled when the app is destroyed.")
   public boolean CancelOnDestroy() {
      return cancelOnDestroy;
   }

   // https://developer.android.com/reference/android/app/Notification.Builder#setSmallIcon(int)
   // ab API Level 11
   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_STRING, defaultValue = "ic_launcher")
   @SimpleProperty(description = "The small icon representing this notification in the status bar and content view.")
   public void SmallSystemIcon(String value) {
      smallSysemIconName = value.trim();
      smallIcon = Util.getSystemIcon(value, form);
   }

   @SimpleProperty(description = "The small icon representing this notification in the status bar and content view.")
   public String SmallSystemIcon() {
      return smallSysemIconName;
   }

   // https://developer.android.com/reference/android/app/Notification.Builder#setSmallIcon(android.graphics.drawable.Icon)
   // ab API Level 11
   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_ASSET, defaultValue = "")
   @SimpleProperty(description = "The small icon representing this notification in the status bar and content view.")
   public void SmallIconImage(String value) {
      smallIconPath = value != null ? value : "";
   }

   @SimpleProperty(description = "The small icon representing this notification in the status bar and content view.")
   public String SmallIconImage() {
      return smallIconPath;
   }

   // https://developer.android.com/reference/android/app/Notification.Builder#setContentTitle(java.lang.CharSequence)
   // ab API Level 11
   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_STRING, defaultValue = "")
   @SimpleProperty(description = "The first bold line of text.")
   public void Title(String value) {
      title = value;
   }

   @SimpleProperty(description = "The first bold line of text.")
   public String Title() {
      return title;
   }

   // https://developer.android.com/reference/android/app/Notification.Builder#setContentText(java.lang.CharSequence)
   // ab API Level 11
   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_STRING, defaultValue = "")
   @SimpleProperty(description = "The second line of text.")
   public void Text(String value) {
      text = value;
   }

   @SimpleProperty(description = "The second line of text.")
   public String Text() {
      return text;
   }

   // https://developer.android.com/reference/android/app/Notification.Builder#setLargeIcon(android.graphics.Bitmap)
   // https://stackoverflow.com/questions/25030710/gcm-push-notification-large-icon-size
   // ab API Level 11
   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_ASSET, defaultValue = "")
   @SimpleProperty(description = "Adds a large icon to the notification content view.")
   public void LargeIcon(String value) {
      largeIconPath = value != null ? value.trim() : "";
   }

   @SimpleProperty(description = "Adds a large icon to the notification content view.")
   public String LargeIcon() {
      return largeIconPath;
   }

   // https://developer.android.com/reference/android/app/Notification.BigPictureStyle
   // https://help.batch.com/en/articles/3993058-what-is-the-best-image-size-for-my-push-notification
   // ab API Level 16
   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_ASSET, defaultValue = "")
   @SimpleProperty(description = "Provides the bitmap to be used as the payload for the BigPicture notification.")
   public void BigPicture(String value) {
      bigPicturePath = value != null ? value.trim() : "";
   }

   @SimpleProperty(description = "Adds a large icon to the notification content view.")
   public String BigPicture() {
      return bigPicturePath;
   }

   // https://developer.android.com/reference/android/app/Notification.Builder#setShowWhen(boolean)
   // ab API Level 11
   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
   @SimpleProperty(description = "Controls whether the timestamp set with setWhen is shown in the content view.")
   public void WhenEnabled(boolean value) {
      showWhen = value;
   }

   @SimpleProperty(description = "Controls whether the timestamp set with setWhen is shown in the content view.")
   public boolean WhenEnabled() {
      return showWhen;
   }

   // https://developer.android.com/reference/android/app/Notification.Builder#setWhen(long)
   // ab API Level 11
   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_NON_NEGATIVE_INTEGER, defaultValue = "0")
   @SimpleProperty(description = "Adds a timestamp pertaining to the notification.")
   public void When(long value) {
      when = value;
   }

   @SimpleProperty(description = "Adds a timestamp pertaining to the notification.")
   public long When() {
      return when;
   }

   // https://developer.android.com/reference/android/app/Notification.Builder#setColor(int)
   // ab API-Level 21
   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_COLOR, alwaysSend = true, defaultValue = Component.DEFAULT_VALUE_COLOR_DEFAULT)
   @SimpleProperty(description = "Accent color to be applied to this notification.")
   public void Color(int argb) {
      color = argb;
   }

   @SimpleProperty(description = "Accent color to be applied to this notification.")
   public int Color() {
      return color;
   }

   // https://developer.android.com/reference/android/app/Notification.Builder#setColorized(boolean)
   // ab API Level 26
   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
   @SimpleProperty(description = "The color set with Color will be used as the background color of this notification.")
   public void Colorized(boolean value) {
      colorized = value;
   }

   @SimpleProperty(description = "The color set with Color will be used as the background color of this notification.")
   public boolean Colorized() {
      return colorized;
   }

   // https://developer.android.com/reference/android/app/Notification.Builder#setUsesChronometer(boolean)
   // ab API Level 16
   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
   @SimpleProperty(description = "Shows the Notification#when field as a stopwatch.")
   public void WhenUsesChronometer(boolean value) {
      usesChronometer = value;
   }

   @SimpleProperty(description = "Shows the Notification#when field as a stopwatch.")
   public boolean WhenUsesChronometer() {
      return usesChronometer;
   }

   // https://developer.android.com/reference/android/app/Notification.Builder#setChronometerCountDown(boolean)
   // ab API Level 24
   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
   @SimpleProperty(description = "Sets the Chronometer to count down instead of counting up.")
   public void WhenChronometerCountDown(boolean value) {
      chronometerCountDown = value;
   }

   @SimpleProperty(description = "Sets the Chronometer to count down instead of counting up.")
   public boolean WhenChronometerCountDown() {
      return chronometerCountDown;
   }

   // Flags
   // https://developer.android.com/reference/android/app/Notification#FLAG_INSISTENT
   // ab API Level 1
   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
   @SimpleProperty(description = "The audio will be repeated until the notification is cancelled or the notification window is opened.")
   public void IsInsistent(boolean value) {
      flagInsistent = value;
   }

   @SimpleProperty(description = "The audio will be repeated until the notification is cancelled or the notification window is opened.")
   public boolean IsInsistent() {
      return flagInsistent;
   }

   // https://developer.android.com/reference/android/app/Notification#FLAG_NO_CLEAR
   // ab API Level 1
   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
   @SimpleProperty(description = "The notification should not be canceled when the user clicks the Clear all button.")
   public void NoClear(boolean value) {
      flagNoClear = value;
   }

   @SimpleProperty(description = "The notification should not be canceled when the user clicks the Clear all button.")
   public boolean NoClear() {
      return flagNoClear;
   }

   // https://developer.android.com/reference/android/app/Notification#FLAG_ONGOING_EVENT
   // ab API Level 1
   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
   @SimpleProperty(description = "This notification is in reference to something that is ongoing, like a phone call.")
   public void IsOngoingEvent(boolean value) {
      flagOngoingEvent = value;
   }

   @SimpleProperty(description = "This notification is in reference to something that is ongoing, like a phone call.")
   public boolean IsOngoingEvent() {
      return flagOngoingEvent;
   }

   // https://developer.android.com/reference/android/app/Notification#FLAG_ONLY_ALERT_ONCE
   // ab API Level 1
   @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
   @SimpleProperty(description = "Sound, vibration and ticker should only be played if the notification has not already been displayed.")
   public void OnlyAlertOnce(boolean value) {
      flagOnlyAlertOnce = value;
   }

   @SimpleProperty(description = "Sound, vibration and ticker should only be played if the notification has not already been displayed.")
   public boolean OnlyAlertOnce() {
      return flagOnlyAlertOnce;
   }

   // https://developer.android.com/reference/android/app/NotificationManager#areNotificationsEnabled()
   // ab API Level 24
   @SimpleProperty(description = "Returns whether notifications are enabled for this app.")
   public boolean AreNotificationsEnabled() {
      if (Build.VERSION.SDK_INT >= 24)
         return notificationManager.areNotificationsEnabled();
      else
         return true;
   }

   // https://developer.android.com/reference/android/app/NotificationManager#areNotificationsPaused()
   // ab API Level 29
   @SimpleProperty(description = "Returns whether notifications are paused for this app.")
   public boolean AreNotificationsPaused() {
      if (Build.VERSION.SDK_INT >= 29)
         return notificationManager.areNotificationsPaused();
      else
         return false;
   }
}
