package de.ullisroboterseite.ursai2notification;

import com.google.appinventor.components.annotations.*;
import com.google.appinventor.components.annotations.androidmanifest.*;
import com.google.appinventor.components.common.*;
import com.google.appinventor.components.runtime.*;
import com.google.appinventor.components.runtime.util.*;

import android.app.*;
import android.content.*;
import android.os.Build;
import android.content.Intent;
import android.content.IntentFilter;
import android.app.Notification.*;

import android.util.Log;

import android.graphics.drawable.Icon;
import android.graphics.Bitmap;
import android.media.AudioAttributes;
import android.media.AudioAttributes.Builder;
import android.media.RingtoneManager;

import android.content.BroadcastReceiver;

import android.text.Spannable;
import android.text.SpannableString;
import android.text.Html;

import java.util.*;
import java.lang.reflect.*;
import java.lang.Thread;

@DesignerComponent(version = 1 //
        , versionName = UrsAI2NotificationChannel.VersionName //
        , dateBuilt = UrsAI2NotificationChannel.dateBuilt //
        , description = "Collection of notification properties." //
        , category = com.google.appinventor.components.common.ComponentCategory.EXTENSION //
        , nonVisible = true //
        , helpUrl = "https://UllisRoboterSeite.de/android-AI2-notifier.html" //
        , iconName = "aiwebres/icon.png")
@SimpleObject(external = true)
public class UrsNotification extends NotificationProperties implements OnDestroyListener {
    final Component thisInstance = this;
    Notification.Builder builder = null;
  
    Notification destroyNotificationObject = null;

    BroadcastReceiver deleteReceiver = new BroadcastReceiver() {
        @Override
        public synchronized void onReceive(final Context arg0, final Intent arg1) {
            int id = arg1.getIntExtra(ID_TAG, -9999);
            if (id == numberID)
                UserCanceled();
        }
    };

    BroadcastReceiver onClickReceiver = new BroadcastReceiver() {
        @Override
        public synchronized void onReceive(final Context arg0, final Intent arg1) {
            int notificationId = arg1.getIntExtra(ID_TAG, -9999);
            String intentId = arg1.getStringExtra(EVENT_TAG);
            if (intentId == null)
                intentId = "";
            if (notificationId == numberID) {
                OnClick(intentId);
            }
        }
    };

    public UrsNotification(ComponentContainer container) {
        super(container);
        Thread.setDefaultUncaughtExceptionHandler(new Thread.UncaughtExceptionHandler() {
            @Override
            public void uncaughtException(Thread paramThread, Throwable paramThrowable) {
                Log.d(LOG_TAG, "Globaler Fehler");
                DebugUtil.LogExecption(paramThrowable);
            }
        });

        notificationManager = (NotificationManager) form.getSystemService(Context.NOTIFICATION_SERVICE);
        form.registerReceiver(deleteReceiver, new IntentFilter(DELETE_TAG));
        form.registerReceiver(onClickReceiver, new IntentFilter(EVENT_TAG));
        form.registerForOnDestroy(this);
    }

    @Override
    public void onDestroy() {
        if (cancelOnDestroy)
            Cancel();

        if (destroyNotificationObject != null) {
            notificationManager.notify(destroyNotification.numberID, destroyNotificationObject);
        }
    }

    @SimpleEvent(description = "Notification was canceled by the user.")
    public void UserCanceled() {
        handler.post(new Runnable() { // Ggf. wird dieses Ereignis in einem Thread ausgelöst.
            public void run() {
                EventDispatcher.dispatchEvent(thisInstance, "UserCanceled");
            }
        });
    }

    @SimpleEvent(description = "The notification or an action button was clicked by the user.")
    public void OnClick(final String IntentID) {
        handler.post(new Runnable() { // Ggf. wird dieses Ereignis in einem Thread ausgelöst.
            public void run() {
                EventDispatcher.dispatchEvent(thisInstance, "OnClick", IntentID);
            }
        });
    }

    // ====================================================================================================================
    @SimpleFunction(description = "Cancels the notification.")
    public void Cancel() {
        notificationManager.cancel(numberID);
    }

    @SimpleFunction(description = "Adds action buttons to the notification.")
    public void AddActionButton(String Title, Component UrsIntentObject) {
        if (!(UrsIntentObject instanceof UrsIntent)) {
            form.ErrorOccurred(this, "AddActionButtons", 17006, "Invalid value at UrsIntentObject");
            return;
        }
        Title = Title.trim();
        if (Title.isEmpty()) {
            form.ErrorOccurred(this, "AddActionButtons", 17010, "Must specify Title");
            return;
        }
        actions.add(new Action(Title.trim(), (UrsIntent) UrsIntentObject));
    }

    @SimpleFunction(description = "Removes all action buttons.")
    public void RemoveActionButtons() {
        actions.clear();
        if (builder != null) {
            try {
                //Use reflection clean up old actions
                Field f = builder.getClass().getDeclaredField("mActions");
                f.setAccessible(true);
                f.set(builder, new ArrayList<Notification.Action>());
                builder.build().actions = new Notification.Action[0];
            } catch (Exception e) {
                // Ersatzweise geht es auch so:
                builder.addAction(0, null, null);
                builder.addAction(0, null, null);
                builder.addAction(0, null, null);
            }
        }
    }

    @SimpleFunction(description = "Submits the Notification.")
    public void Create(Component UrsChannelObject, Component UrsIntentObject) {
        if (!(UrsChannelObject instanceof UrsAI2NotificationChannel)) {
            form.ErrorOccurred(this, "Create", 17001, "Invalid value at UrsChannelObject");
            return;
        }

        if (!(UrsIntentObject instanceof UrsIntent)) {
            form.ErrorOccurred(this, "Create", 17006, "Invalid value at UrsIntentObject");
            return;
        }

        UrsAI2NotificationChannel uco = (UrsAI2NotificationChannel) UrsChannelObject;
        UrsIntent uio = (UrsIntent) UrsIntentObject;
        PendingIntent pendingIntent = uio.buildIntent(numberID);
        notificationManager.notify(numberID,
                buildNotification(uco.channelID, uco.importance, pendingIntent, uco.soundEnabled, -1, -1, false));
    }

    Notification CreateDestroyNotification(Component UrsChannelObject, Component UrsIntentObject) {
        UrsAI2NotificationChannel uco = (UrsAI2NotificationChannel) UrsChannelObject;
        UrsIntent uio = (UrsIntent) UrsIntentObject;
        PendingIntent pendingIntent = uio.buildIntent(numberID);
        return buildNotification(uco.channelID, uco.importance, pendingIntent, uco.soundEnabled, -1, -1, false);
    }

    @SimpleFunction(description = "Defines the replacemant for the notification when the Screen is destroyed.")
    public void SetOnDestroyAction(Component UrsChannelObject, Component UrsNotificationObject,
            Component UrsIntentObject) {
        if (!(UrsChannelObject instanceof UrsAI2NotificationChannel)) {
            form.ErrorOccurred(this, "SetOnDestroyAction", 17001, "Invalid value at UrsChannelObject");
            return;
        }

        if (!(UrsIntentObject instanceof UrsIntent)) {
            form.ErrorOccurred(this, "SetOnDestroyAction", 17006, "Invalid value at UrsIntentObject");
            return;
        }

        if (!(UrsNotificationObject instanceof UrsNotification)) {
            form.ErrorOccurred(this, "SetOnDestroyAction", 17005, "Invalid value at UrsNotificationObject");
            return;
        }

        destroyChannel = (UrsAI2NotificationChannel) UrsChannelObject;
        destroyNotification = (UrsNotification) UrsNotificationObject;
        destroyIntent = (UrsIntent) UrsIntentObject;

        destroyNotificationObject = destroyNotification.CreateDestroyNotification(UrsChannelObject, UrsIntentObject);
    }

    @SimpleFunction(description = "Removes the OnDestroyAction.")
    public void RemoveOnDestroyAction() {
        destroyChannel = null;
        destroyNotification = null;
        destroyIntent = null;

        destroyNotificationObject = null;
    }

    @SimpleFunction(description = "Adjust the notification progress bar.")
    public void SetProgressEx(String Title, String Text, int CurrentProgress) {
        if (builder == null) {
            form.ErrorOccurred(this, "SetProgressEx", 17009, "Notification not created yet.");
            return;
        }
        builder.setProgress(maxProgress, CurrentProgress, false);
        SpannableString spannedTitle = new SpannableString(Html.fromHtml(Title));
        builder.setContentTitle(spannedTitle);
        SpannableString spannedText = new SpannableString(Html.fromHtml(Text));
        builder.setContentText(spannedText);
        notificationManager.notify(numberID, builder.build());
    }

    @SimpleFunction(description = "Adjust the notification progress bar.")
    public void SetProgress(int CurrentProgress) {
        if (builder == null) {
            form.ErrorOccurred(this, "SetProgress", 17009, "Notification not created yet.");
            return;
        }

        builder.setProgress(maxProgress, CurrentProgress, false);
        notificationManager.notify(numberID, builder.build());
    }

    @SimpleFunction(description = "Remove the notification progress bar.")
    public void RemoveProgressBarEx(String Title, String Text) {
        if (builder == null) {
            form.ErrorOccurred(this, "RemoveProgressBarEx", 17009, "Notification not created yet.");
            return;
        }
        builder.setProgress(0, 0, false);
        SpannableString spannedTitle = new SpannableString(Html.fromHtml(Title));
        builder.setContentTitle(spannedTitle);
        SpannableString spannedText = new SpannableString(Html.fromHtml(Text));
        builder.setContentText(spannedText);

        notificationManager.notify(numberID, builder.build());
    }

    @SimpleFunction(description = "Remove the notification progress bar.")
    public void RemoveProgressBar() {
        if (builder == null) {
            form.ErrorOccurred(this, "RemoveProgressBar", 17009, "Notification not created yet.");
            return;
        }

        builder.setProgress(0, 0, false);
        notificationManager.notify(numberID, builder.build());
    }

    int getFlags() {
        int flags = 0;
        if (flagNoClear)
            flags += Notification.FLAG_NO_CLEAR;
        if (flagInsistent)
            flags += Notification.FLAG_INSISTENT;
        if (flagOngoingEvent)
            flags += Notification.FLAG_ONGOING_EVENT;
        if (flagOnlyAlertOnce)
            flags += Notification.FLAG_ONLY_ALERT_ONCE;
        return flags;
    }

    Notification buildNotification(String channelID, int importance, PendingIntent pendingIntent, boolean soundEnabled,
            int currentProgress, int maxProgress, boolean indeterminate) {

        Notification.Builder nb = getNotificationBuilder(form, channelID);
        builder = nb;

        nb.setContentIntent(pendingIntent);
        nb.setLargeIcon(Util.getBitmapFromAsset((Context) form, largeIconPath));

        SpannableString spannedTitle = new SpannableString(Html.fromHtml(title));
        nb.setContentTitle(spannedTitle);

        SpannableString spannedText = new SpannableString(Html.fromHtml(text));
        nb.setContentText(spannedText);

        nb.setShowWhen(showWhen);
        if (when > 0)
            nb.setWhen(when);
        nb.setAutoCancel(autoCancel);

        if (Build.VERSION.SDK_INT <= 25)
            nb.setPriority(importance);

        if (Build.VERSION.SDK_INT >= 16) {
            nb.setUsesChronometer(usesChronometer);
        }

        if (Build.VERSION.SDK_INT >= 16) {
            if (!bigPicturePath.isEmpty()) {
                nb.setStyle(
                        new Notification.BigPictureStyle().bigPicture(Util.getBitmapFromAsset(form, bigPicturePath)));
            }
        }

        if (Build.VERSION.SDK_INT >= 24)
            nb.setChronometerCountDown(chronometerCountDown);
        if (Build.VERSION.SDK_INT >= 21)
            if (color != Component.COLOR_DEFAULT)
                nb.setColor(color);
        if (Build.VERSION.SDK_INT >= 26)
            nb.setColorized(colorized);

        if (Build.VERSION.SDK_INT >= 21 && Build.VERSION.SDK_INT <= 26)
            if (soundEnabled) {
                AudioAttributes audioAttributes = new AudioAttributes.Builder() //
                        .setContentType(AudioAttributes.CONTENT_TYPE_SONIFICATION) //
                        .setUsage(AudioAttributes.USAGE_NOTIFICATION) //
                        .build();
                nb.setSound(RingtoneManager.getDefaultUri(RingtoneManager.TYPE_NOTIFICATION), audioAttributes);
            }

        Icon iconObject = Util.getIconFromAssets(form, smallIconPath);
        if (iconObject != null)
            nb.setSmallIcon(iconObject);
        else
            nb.setSmallIcon(smallIcon);

        if (currentProgress >= 0) {
            this.maxProgress = maxProgress;
            nb.setProgress(maxProgress, currentProgress, indeterminate);
        }

        for (Action action : actions) {
            PendingIntent pe = action.ursIntent.buildIntent(numberID);
            SpannableString spannedActionTitle = new SpannableString(Html.fromHtml(action.title));
            nb.addAction(0, spannedActionTitle, pe); //Kein Icon. Icon wird nur bei älteren Versionen unterstützt
        }

        Intent deleteIntent = new Intent(DELETE_TAG);
        deleteIntent.putExtra(ID_TAG, numberID);

        PendingIntent peDelete;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M)
            peDelete = PendingIntent.getBroadcast(form, 0, deleteIntent,
                    PendingIntent.FLAG_UPDATE_CURRENT | PendingIntent.FLAG_IMMUTABLE);
        else
            peDelete = PendingIntent.getBroadcast(form, 0, deleteIntent, PendingIntent.FLAG_UPDATE_CURRENT);

        nb.setDeleteIntent(peDelete);

        Notification notification = nb.build();
        notification.flags += getFlags();

        return notification;
    }

    public Intent addPropertiesToIntent(String channelID, int importance, Intent intent) {
        intent.putExtra("ChannelID", channelID);
        intent.putExtra("NumberID", numberID);
        intent.putExtra("Priority", importance);
        intent.putExtra("SmallIcon", smallIcon);
        intent.putExtra("SmallIconPath", smallIconPath);
        intent.putExtra("LargeIconPath", largeIconPath);
        intent.putExtra("BigPicturePath", bigPicturePath);
        intent.putExtra("Title", title);
        intent.putExtra("Text", text);
        intent.putExtra("ShowWhen", showWhen);
        intent.putExtra("When", when);
        intent.putExtra("AutoCancel", autoCancel);
        intent.putExtra("UsesChronometer", usesChronometer);
        intent.putExtra("ChronometerCountDown", chronometerCountDown);
        intent.putExtra("Color", color);
        intent.putExtra("Colorized", colorized);
        intent.putExtra("Flags", getFlags());
        intent.putExtra("ActionSize", actions.size());
        for (int i = 0; i < actions.size(); i++) {
            intent.putExtra("ActionTitle" + i, actions.get(i).title);
            PendingIntent pe = actions.get(i).ursIntent.buildIntent(numberID);
            intent.putExtra("ActionIntent" + i, pe);
        }
        return intent;
    }

    /**
     * Erstellt eine Notification aus den Extra-Daten des AlarmIntent
     * @param context Der Context des Alarmreceivers
     * @param dataIntent Der Intent mit den Extra-Daten
     * @return Die erstellte Notification.
     */
    public static Notification buildFromIntentExtras(Context context, Intent dataIntent) {
        PendingIntent pendingIntent = UrsIntent.buildFromIntentExtras(context, dataIntent);

        Notification.Builder nb = getNotificationBuilder(context, dataIntent.getStringExtra("ChannelID"));

        nb.setContentIntent(pendingIntent);
        SpannableString spannedTitle = new SpannableString(Html.fromHtml(dataIntent.getStringExtra("Title")));
        nb.setContentTitle(spannedTitle);
        SpannableString spannedText = new SpannableString(Html.fromHtml(dataIntent.getStringExtra("Text")));
        nb.setContentText(spannedText);

        nb.setShowWhen(dataIntent.getBooleanExtra("ShowWhen", false));
        nb.setWhen(dataIntent.getLongExtra("When", 0));
        nb.setAutoCancel(dataIntent.getBooleanExtra("AutoCancel", false));

        if (Build.VERSION.SDK_INT >= 16) {
            nb.setUsesChronometer(dataIntent.getBooleanExtra("UsesChronometer", false));
            String bigPicturePath = dataIntent.getStringExtra("BigPicturePath");
            if (!bigPicturePath.isEmpty())
                nb.setStyle(new Notification.BigPictureStyle()
                        .bigPicture(Util.getBitmapFromAsset(context, bigPicturePath)));
        }

        if (Build.VERSION.SDK_INT >= 24)
            nb.setChronometerCountDown(dataIntent.getBooleanExtra("ChronometerCountDown", false));
        if (Build.VERSION.SDK_INT >= 21) {
            int color = dataIntent.getIntExtra("Color", 0);
            if (color != Component.COLOR_DEFAULT)
                nb.setColor(color);
        }

        if (Build.VERSION.SDK_INT <= 25)
            nb.setPriority(dataIntent.getIntExtra("Priority", 0));

        if (Build.VERSION.SDK_INT >= 26)
            nb.setColorized(dataIntent.getBooleanExtra("Colorized", false));

        nb.setLargeIcon(Util.getBitmapFromAsset(context, dataIntent.getStringExtra("LargeIconPath")));

        Icon iconObject = Util.getIconFromAssets(context, dataIntent.getStringExtra("SmallIconPath"));
        if (iconObject != null)
            nb.setSmallIcon(iconObject);
        else
            nb.setSmallIcon(dataIntent.getIntExtra("SmallIcon", 0));

        int actionSize = dataIntent.getIntExtra("ActionSize", 0);
        for (int i = 0; i < actionSize; i++) {
            PendingIntent pe = (PendingIntent) dataIntent.getParcelableExtra("ActionIntent" + i);
            String title = dataIntent.getStringExtra("ActionTitle" + i);
            if (title == null)
                continue;
            nb.addAction(0, title, pe);
        }

        Notification notification = nb.build();
        notification.flags += dataIntent.getIntExtra("Flags", 0);
        return notification;
    }

    static Notification.Builder getNotificationBuilder(Context context, String channelID) {
        if (Build.VERSION.SDK_INT < 26) { // Vor Version 26 gab es keine Kanäle
            return new Notification.Builder(context);
        }
        return new Notification.Builder(context, channelID);
    }

    static Notification buildSimpleNotification(Form form, String channelID, int numberID, String iconName,
            String title, String text, Boolean autoCancel, PendingIntent pendingIntent) {

        Notification.Builder nb = getNotificationBuilder(form.$context(), channelID);

        SpannableString spannedTitle = new SpannableString(Html.fromHtml(title));
        nb.setContentTitle(spannedTitle);
        SpannableString spannedText = new SpannableString(Html.fromHtml(text));
        nb.setContentText(spannedText);

        nb.setContentIntent(pendingIntent);
        nb.setAutoCancel(autoCancel);

        Icon iconObject = Util.getIconFromAssets(form, iconName);

        if (iconObject == null)
            nb.setSmallIcon(Util.getSystemIcon(iconName, form));
        else
            nb.setSmallIcon(iconObject);

        return nb.build();
    }
}